
/**
 * Module dependencies.
 */

var boot = require('./boot')
  , Component = require('./Component')
  , debug = require('debug')('component:middleware')
  , join = require('path').join
  , fs = require('fs');

/**
 * Expose modules in `dir` as middleware,
 * and compile output to the given `dest` directory.
 *
 * @param {String} dir
 * @param {String} dest
 * @param {Function} fn
 * @return {Function}
 * @api public
 */

module.exports = function(dir, dest, fn){
  boot.scripts(function(err, str){
    if (err) throw err;
    var path = join(dest, 'boot.js');
    fs.writeFile(path, str);
    debug('built %s', path);
  });

  return function(req, res, next){
    var m;
    if (m = req.url.match(/^\/component\/([-\w]+)\.(js|css)/)) {
      var name = m[1];
      var type = m[2];
      var method = type == 'css'
        ? 'buildStyles'
        : 'buildScripts';

      // TODO: fix massive lookup hack
      var path = join(dir, name, 'package.json');
      debug('building %s %s', name, type);
      debug('path: %s', path);
      var pkg = new Component(path);

      if (fn) fn(pkg);

      path = join(dest, name)
      debug('dest: %s', path);
      pkg[method](path, function(err){
        if (err) return next(err);
        req.url = '/' + name + '.' + type;
        debug('rewrite url to %s', req.url);
        next();
      }, false);
      return;
    }

    next();
  }
};