
/**
 * Module dependencies.
 */

var debug = require('debug')('component:compress');

/**
 * JavaScript and CSS compression plugin.
 *
 * Options:
 *
 *   - `js` enable js compression [true]
 *   - `css` enable css compression [true]
 *
 * @param {Object} options
 * @return {Function}
 * @api public
 */

module.exports = function(options){
  options = options || {};
  var compressjs = options.js !== false
    , compresscss = options.css !== false
    , uglifyjs
    , uglifycss;

  if (compressjs) uglifyjs = require('uglify-js');
  if (compresscss) uglifycss = require('uglifycss');

  debug('used compress()');

  return function(component){
    component.hook('styles', function(css){
      if (compresscss) return uglifycss.processString(css);
      return css;
    });

    component.hook('scripts', function(str){
      if (compressjs) return uglifyjs.uglify.gen_code(uglifyjs.parser.parse(str));
      return str;
    });
  }
}