
/**
 * Module dependencies.
 */

var debug = require('debug')('component:rework');
var whitespace = require('css-whitespace');

/**
 * Prefixed properties.
 */

var props = [
  'animation',
  'animation-delay',
  'animation-direction',
  'animation-duration',
  'animation-fill-mode',
  'animation-iteration-count',
  'animation-name',
  'animation-play-state',
  'animation-timing-function',
  'appearance',
  'background-visibility',
  'background-clip',
  'background-composite',
  'background-origin',
  'background-size',
  'blend-mode',
  'border-bottom-left-radius',
  'border-bottom-right-radius',
  'border-fit',
  'border-image',
  'border-radius',
  'border-top-left-radius',
  'border-top-right-radius',
  'border-vertical-spacing',
  'box-align',
  'box-direction',
  'box-flex',
  'box-flex-group',
  'box-lines',
  'box-ordinal-group',
  'box-orient',
  'box-pack',
  'box-reflect',
  'box-shadow',
  'box-sizing',
  'clip-path',
  'column-count',
  'column-width',
  'column-min-width',
  'column-width-policy',
  'column-gap',
  'column-rule',
  'column-rule-color',
  'column-rule-style',
  'column-rule-width',
  'column-span',
  'flex',
  'flex-basis',
  'flex-direction',
  'flex-flow',
  'flex-grow',
  'flex-shrink',
  'flex-wrap',
  'flex-flow-from',
  'flex-flow-into',
  'font-smoothing',
  'transform',
  'transform-origin',
  'transform-origin-x',
  'transform-origin-y',
  'transform-origin-z',
  'transform-style',
  'transition',
  'transition-delay',
  'transition-duration',
  'transition-property',
  'transition-timing-function',
  'user-drag',
  'user-modify',
  'user-select',
  'wrap',
  'wrap-flow',
  'wrap-margin',
  'wrap-padding',
  'wrap-through'
];

/**
 * Vendors supported.
 */

var vendors = ['-ms-', '-moz-', '-webkit-'];

/**
 * Rework css plugin.
 *
 * @return {Function}
 * @api public
 */

module.exports = function(){
  var rework = require('rework');
  var mixins = require('rework-mixins');
  var vars = require('rework-vars');
  var css;

  debug('used rework()');

  function reset() {
    debug('reset');
    css = '';
  }

  reset();

  return function(component){
    component.on('reset', reset);

    component.on('stylesheet:styl', function(path, str, fn){
      css += whitespace(str);
      fn();
    });

    component.hook('styles', function(other){
      return rework(css)
        .vendors(vendors)
        .use(rework.extend())
        .use(vars())
        .use(rework.colors())
        .use(rework.references())
        .use(rework.keyframes())
        .use(rework.ease())
        .use(rework.prefixValue('transform'))
        .use(rework.prefixValue('linear-gradient'))
        .use(rework.prefix(props))
        .use(rework.mixin(mixins))
        .use(rework.at2x())
        .toString() + other;
    });
  };
};
